#include <windows.h>
#include <stdio.h>


char get_bits (char *name);

void get_palette (RGBQUAD palette[256], bool pal2p);

bool convert (char *name, char bits, RGBQUAD palette[256]);


int main (int argc, char *argv[])
{
	RGBQUAD palette[256];

	// J[pbg擾
	if ((argc > 1) && (strcmp (argv[1], "-2p") == 0))
		get_palette (palette, true);
	else
		get_palette (palette, false);

	HANDLE hFind = NULL;
	WIN32_FIND_DATA wfd;

	if ((hFind = FindFirstFile ("*.cnv", &wfd)) != INVALID_HANDLE_VALUE) {
		do {
			printf ("%s -> ", wfd.cFileName);

			// 摜`FbNiFATCYj
			char bits;
			if ((bits = get_bits (wfd.cFileName)) < 0) {
				printf ("cnv convert failed.\n");
				continue;
			}

			if (!convert (wfd.cFileName, bits, palette)) {
				printf ("cnv convert failed.\n");
				continue;
			}

			printf ("OK\n");
		} while (FindNextFile (hFind, &wfd));
	} else {
		printf ("cnv file search failed.\n");
		return 0;
	}

	FindClose (hFind);

	return 0;
}


char get_bits (char *name)
{
	FILE *cnv = fopen (name, "rb");
	if (cnv == NULL) return -1;

	char bits;
	int width, height, pitch;
	fread (&bits, 1, 1, cnv);
	fread (&width, 4, 1, cnv);
	fread (&height, 4, 1, cnv);
	fread (&pitch, 4, 1, cnv);

	fclose (cnv);

	// Ή̐F
	if ((bits != 8) && (bits != 16) && (bits != 24) && (bits != 32))
		return -1;

	// Ή̃TCYiwi摜ł1400*1200Oj
	if ((width > 2000) || (height > 2000) || (width < 0) || (height < 0))
		return -1;

	return bits;
}


void get_palette (RGBQUAD palette[256], bool pal2p)
{
	FILE *pal = NULL;
	WORD buf[256];
	memset (palette, 0, 1024);

	// 1Ppbgf[^擾
	pal = fopen ((pal2p) ? "palette001.pal" : "palette000.pal", "rb");
	if (pal == NULL) {
		palette[0].rgbReserved = 0xFF;
		return;
	}
	fseek (pal, 1, SEEK_SET);
	fread (buf, 2, 256, pal);
	fclose (pal);

	// pbgf[^16bits24bitsɕϊ
	for (int i = 0; i < 256; i++) {
		float r, g, b;
		b = (float)(buf[i] & 0x1F);
		g = (float)((buf[i] & 0x3E0) >> 5);
		r = (float)((buf[i] & 0x7C00) >> 10);

		palette[i].rgbBlue = (byte)(b *8.25);
		palette[i].rgbGreen = (byte)(g *8.25);
		palette[i].rgbRed = (byte)(r *8.25);
	}
}


bool convert (char *name, char bits, RGBQUAD palette[256])
{
	if ((bits == 8) && (palette[0].rgbReserved == 0xFF))
		return false;

	int bytes = ((bits == 24) ? 4 : bits/8);

	FILE *cnv = fopen (name, "rb");

	int width, height, pitch;
	fseek (cnv, 1, SEEK_SET);
	fread (&width, 4, 1, cnv);
	fread (&height, 4, 1, cnv);
	fread (&pitch, 4, 1, cnv);
	fseek (cnv, 0x11, SEEK_SET);

	char *buf = new char[pitch*height*bytes];
	fread (buf, 1, pitch*height*bytes, cnv);

	fclose (cnv);

	int hdr = 0x36+((bytes == 1) ? 0x400 : 0);

	BITMAPFILEHEADER bf = {0x4D42, pitch*height*bytes+hdr, 0, 0, hdr};
	BITMAPINFOHEADER bi = {0x28, pitch, height, 1, bytes*8, BI_RGB, pitch*height*bytes, 0, 0, 0, 0};

	// o̓t@C쐬
	char sbmp[128];
	strcpy (sbmp, name);
	strcpy (strrchr (sbmp, '.'), ".bmp");

	FILE *bmp = fopen (sbmp, "wb");
	if (bmp == NULL) {
		delete [] buf;
		return false;
	}

	fwrite (&bf, 1, sizeof (BITMAPFILEHEADER), bmp);
	fwrite (&bi, 1, sizeof (BITMAPINFOHEADER), bmp);
	if (bits == 8) fwrite (palette, 4, 256, bmp);
	for (int i = height-1; i >= 0; i--)
		fwrite (&buf[i*pitch*bytes], 1, pitch*bytes, bmp);
	fclose (bmp);

	delete [] buf;

	return true;
}
